#include <xtl/fiber.hpp>
#include <xtl/scheduler.hpp>

#if defined(DEBUG_FIBER) || defined(DEBUG_ALL)
#define fdebug(fmt, ...) fprintf(stderr, "fiber[%d]: " fmt "\n", __LINE__, __VA_ARGS__);
#define debug(msg) fprintf(stderr, "fiber[%d]: " msg "\n", __LINE__);
#else
#define fdebug(fmt, ...)
#define debug(msg)
#endif

namespace xtl {

class fiber_impl {
public:
	inline fiber_impl(xtl::function<void()> & function, size_t stack_size)
	{
		scheduler::create_fiber(this, std::move(function), stack_size);
		fdebug("Created fiber_impl %p", (void *) this);
	}

	inline ~fiber_impl() noexcept {
		fdebug("Destroying fiber_impl %p", (void *) this);
		scheduler::destroy_fiber(this);
	}
};

fiber::fiber(xtl::function<void()> function, size_t stack_size)
: shared_ptr(std::make_shared<xtl::fiber_impl>(function, stack_size))
{}

} // namespace xtl
